import static org.junit.Assert.*;
import org.junit.Test;
import org.junit.FixMethodOrder;
import org.junit.runners.MethodSorters;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;

/**
* A class with unit tests for a MyPriorityQueue implementation.
* It tests the MyArrayPriorityQueue class but is designed to be able to
* be extended for testing other MyPriorityQueue implementations.
* For that, subclasses need only override the createEmptyLongQueue() 
* method.
*
* The tests try to test all methods and corner cases.
* We recommend you tackle them (i.e. try to make them pass) in the 
* order they appear in this file.
*
* @author  Rui Meireles
* @version 1.0
*/
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class MyArrayPiorityQueueTest extends AMyCollectionTest{

    protected MyCollection<Long> createEmptyLongCollection(){
        return createEmptyLongQueue();
    }

    /**
     * A factory method to create an empty queue of long integers using the 
     * implementation of My Queue we're trying to test.
     * 
     * @return an empty long list of integers.
     */
    protected MyPriorityQueue<Long> createEmptyLongQueue(){
        return new MyArrayPriorityQueue<Long>();
    }

    /**
     * Tests whether the min-priority queue removal order is respected.
     * Assumes correctness of the following methods: add(), size(), remove(E elem).
     */
    @Test
    public void testQueue1AddAndRemove(){
        MyPriorityQueue<Long> queue = createEmptyLongQueue(); // new empty list        
        assertEquals("Queue remove from empty queue doesn't yield null", null, queue.remove());
        assertEquals("Empty queue has non-zero length", 0, queue.size());

        // create a list of longs
        List<Long> longList = new ArrayList();
        for (long i=1; i <= NINTS; i++) longList.add(new Long(i));

        for (int j=0; j < 20; j++){ // repeat the test 20 times
            Collections.shuffle(longList);
            for (int i=0; i < NINTS; i++) // add all them items, in random order
                queue.add(longList.get(i));

            // remove all items and check that they come out in the right order
            for (long i=1; i <= NINTS; i++){ // add all them items, in random order
                Long ogElem = new Long(i);
                
                if (Math.random() >= 0.5){ // flip a coin to either test remove()...
                    Long remElem = queue.remove();
                    assertEquals("Element not removed from queue in min-priority order", ogElem, remElem);
                } else { // or that remove(E elem) preserve the remaining elements' order 
                    assertEquals("Element that should be in queue couldn't be removed from it", true, queue.remove(ogElem));                
                }
                assertEquals("Queue size incorrect after element removal", NINTS-i, queue.size());
            }
            assertEquals("Queue remove from empty queue doesn't yield null", null, queue.remove());
            assertEquals("Queue size non-zero after all elements have been removed", 0, queue.size());
        }   
    }

}