/**
 * An interface specifying the operations that should be supported
 * by a worklist abstract data type.
 * Worklists are collections characterized by having a rule that
 * dictates the order in which elements should be removed.
 *
 * @author Rui Meireles
 * @version 1.0
 */
public interface MyWorklist<E> extends MyCollection<E>{

    /**
     * Adds an element to the worklist.
     *
     * @param elem the element to be added.
     * @return true if the collection changed as a result of this addition; 
     * false otherwise.
     */
    boolean add(E elem);
    
    /**
     * Removes the next item to be removed from the worklist, 
     * according to its removal rule.
     * 
     * @return the item removed, or null if queue is empty.
     */
    E remove();
    
    /**
     * Retrieves the next item to be removed from the worklist, 
     * according to its removal rule, without actually removing it.
     * 
     * @return the next item to be removed, or null if queue is empty.
     */
    E peek();
}
